var Constants = {
    NoEntity: -1,
    NoResourceId: -1,
    NoType: 0xFFFF,
};

var DynamicComponent = function () {

};

var lowercaseFirstLetter = function (string) {
    return string.charAt(0).toLowerCase() + string.slice(1);
}

DynamicComponent.prototype.constructor = DynamicComponent;

var numTypes = 0;

var ComponentCollection = {
    create: function (runtime, entity) {
        var typeInfos = {};

        var getDynamicTypeInfo = function (component) {
            var isDynamic = component instanceof DynamicComponent;
            var prototypeInfo = Object.getPrototypeOf(component).typeInfo;

            if (isDynamic && prototypeInfo) {
                typeInfos[component] = prototypeInfo;
                return prototypeInfo;
            }

            var typeInfo = {
                messagesById: {},
                mask: 0,
                typeId: -1,
            };

            for (var prop in component) {
                if (typeof component[prop] == "function") {
                    var messageId = runtime.bridge.GetMessageId(prop);

                    if (messageId != -1) {
                        typeInfo.mask |= 1 << messageId;
                        typeInfo.messagesById[messageId] = prop;
                    }
                }
            }

            var name = "JsComponent" + numTypes++;

            runtime.bridge.RegisterDynamicType(name, typeInfo.mask);
            typeInfo.typeId = runtime.bridge.getTypeId(name);

            typeInfos[component] = typeInfo;

            if (isDynamic) {
                Object.getPrototypeOf(component).typeInfo = typeInfo;
            }

            return typeInfo;
        };

        var components = new Array();
        return Object.assign(components, {
            add: function (component) {
                if (component.isTyped) {
                    ComponentHandler.addComponent(runtime, entity, component);

                    components.push(component);

                    entity[lowercaseFirstLetter(component.name)] = component;

                    return component;
                }

                var typeInfo = getDynamicTypeInfo(component);

                var dispatch = function (id, data) {
                    try {
                        component[typeInfo.messagesById[id]](data);
                    } catch (e) {
                        console.log("Dispatch error: " + e);
                    }
                };

                runtime.bridge.dispathcers = runtime.bridge.dispathcers || [];
                runtime.bridge.dispathcers.push(dispatch);

                runtime.bridge.AddComponent(entity, typeInfo.typeId);

                runtime.bridge.RegisterMessageCallback(entity, typeInfo.typeId, dispatch);

                components.push(component);

                return component;
            },
            remove: function (component) {
                var index = components.indexOf(component);

                if (index == -1) return;

                var typeId = typeInfos[component].typeId;

                runtime.bridge.RegisterMessageCallback(entity, typeId, null);
                runtime.bridge.RemoveComponent(entity, typeId);

                typeInfos[component] = null;

                components.splice(index, 1);
            }
        });
    }
};
