local Entity = {}
function Entity.create(bridge, type, name)
	local entity = {
		id = bridge.createEntity(bridge.context, type),
		name = name,
	}

	if name then
		bridge.setEntityName(bridge.context, entity.id, name)
	end

	return entity
end

Component = Object.newClass("Component")

EntityHandler = {}
EntityHandler.__index = EntityHandler

function EntityHandler.new(runtime)
	local entityHandler = {
		runtime = runtime,
		bridge = runtime.bridge
	}

	setmetatable(entityHandler, EntityHandler)

	return entityHandler
end

function EntityHandler:reflectEntity(entity)
	local bridge = self.runtime.bridge
	local context = bridge.context

	local componentTypes = {}
	local numComponents = bridge.getComponents(context, entity.id, componentTypes);

	for i=0,numComponents-1 do
		local typeId = componentTypes[i]

		local componentName = bridge.getTypeName(typeId)

		local lowercase = string.lower(string.sub(componentName, 1, 1)) .. string.sub(componentName, 2)

		local component = Component.new()

		entity[lowercase] = component

		local numFields = bridge.getNumFields(typeId);

		for i=0,numFields-1 do
			local fieldName = bridge.getFieldName(typeId, i);
			local fieldType = bridge.getFieldType(typeId, i);

			local field = {
				value = nil,
				fieldType = fieldType,
				setter = self.runtime.settersById[fieldType]
			}

			if field.setter == nil then
				console.warn("Failed to get setter for field " .. fieldName)
			end

			Object.defineProperty(component, fieldName, {
				get = function ()
					return field.value
				end,
				set = function (value)
					field.value = value

					if field.setter then
						field.setter(context, entity.id, typeId, i, value)
					end
				end
			})
		end
	end
end

function EntityHandler:createEntityFactory(type)
	return function (name)
		local entity = Entity.create(self.bridge, type, name)

		entity.components = ComponentCollection.new(self.bridge, entity)

		self:reflectEntity(entity)

		return entity
	end
end

function EntityHandler:initializeEntities()
	local entities = {
		"Camera",
		"Light",
		"Cube",
		"Sphere",
		"Cylinder",
		"Plane",
		"MeshPart",
		"Group",
		"Lod",
		"MeshPartWithMaterial",
		"ModelEntity",
		"SkyDome",
		"HeightMap",
		"GenericShape",
		"Trajectory",
		"DepthAxis",
		"AxisCube",
		"CubeMarker",
		"DataSet",
		"Extrusion",
		"Wellbore",
		"WellLog",
		"LookupColorMap",
		"Casing",
		"DataSet2D",
		"OceanRectangle"
	}

	local bridge = self.runtime.bridge
	local context = bridge.context

	for key,value in pairs(entities) do
		runtime[value] = {
			create = self:createEntityFactory(value)
		}
	end

    runtime.Shape = runtime.GenericShape;
    runtime.Log = runtime.WellLog;
    runtime.ColorMap = runtime.LookupColorMap;
    runtime.BasicOcean = runtime.OceanRectangle;
end
