#include "Noise.es20.glsl"


vec3 decodeDisplacement(vec3 value)
{
#ifdef COGS_BASIC_OCEAN_ENCODING_SCALED
  return valueScale().x*(value - vec3(0.5));
#else
  return value;
#endif
}

vec4 decodeTangents(vec4 value)
{
#ifdef COGS_BASIC_OCEAN_ENCODING_SCALED
  return valueScale().y*(value - vec4(0.5));
#else
  return value;
#endif
}


struct WaveParameters
{
  vec2 b;
  vec2 phase;
};

WaveParameters calculateWaveParameters(vec2 world)
{
  WaveParameters w;
  w.b = fftTileScale() * world;

  float theta = 2.1 * pnoise(phaseShiftNoiseFrequency() * w.b, vec2(phaseShiftNoisePeriod()));
  w.phase = vec2(cos(theta), sin(theta));

  return w;
}

vec3 waveDisplace(vec2 world)
{
  WaveParameters w = calculateWaveParameters(world);

#ifdef COGS_VERTEX_SHADER
  vec3 real = decodeDisplacement(texture2DLod(Displacement, w.b, 0.0).rgb);
  vec3 imag = decodeDisplacement(texture2DLod(Difference, w.b, 0.0).rgb);
#else
  vec3 real = decodeDisplacement(texture2DLodEXT(Displacement, w.b, 0.0).rgb);
  vec3 imag = decodeDisplacement(texture2DLodEXT(Difference, w.b, 0.0).rgb);
#endif


  float z = -(real.z * w.phase.x - imag.z * w.phase.y);
  vec2 xy = real.xy * w.phase.y + imag.xy * w.phase.x;

  return significantWaveHeight() * vec3(xy, z);
}

vec3 waveNormal(vec2 world)
{
  WaveParameters w = calculateWaveParameters(world);

  vec4 tmp_a = decodeTangents(texture2D(Tangents, w.b));
  vec4 tmp_b = decodeTangents(texture2D(Tangents, 16.0 * w.b));
  vec4 tmp = significantWaveHeight() * (tmp_a + 0.5 * tmp_b);

  vec2 rxy = (tmp.xz * w.phase.x - tmp.yw * w.phase.y); // Not completely correct, but seems to increase sharpness a bit
  vec2 xy = tmp.xz * w.phase.y + tmp.yw * w.phase.x;

  vec3 du = vec3(1.0 + rxy.x, 0.0, xy.x);
  vec3 dv = vec3(0.0, 1.0 + rxy.y, xy.y);
  vec3 m = cross(du, dv);

  return normalize(vec3(waveDirection().x * m.x + waveDirection().y * m.y,
                       -waveDirection().y * m.x + waveDirection().x * m.y,
                        m.z));
}
