#include "Noise.hlsl"

struct WaveParameters
{
  float2 b;
  float2 phase;
};

float3 decodeDisplacement(float3 value)
{
#ifdef COGS_BASIC_OCEAN_ENCODING_SCALED
  return valueScale.x * (value - (float3)0.5);
#else
  return value;
#endif
}

float4 decodeTangents(float4 value)
{
#ifdef COGS_BASIC_OCEAN_ENCODING_SCALED
  return valueScale.y * (value - (float4)0.5);
#else
  return value;
#endif
}


WaveParameters calculateWaveParameters(float2 world)
{
  WaveParameters w;
  w.b = fftTileScale * world;

  float theta = 2.1 * pnoise(phaseShiftNoiseFrequency * w.b, (float2)(phaseShiftNoisePeriod));
  w.phase = float2(cos(theta), sin(theta));

  return w;
}

float3 waveDisplace(float2 world)
{
  WaveParameters w = calculateWaveParameters(world);

  float3 real = decodeDisplacement(Displacement.SampleLevel(DisplacementSampler, w.b, 0).rgb);
  float3 imag = decodeDisplacement(Difference.SampleLevel(DifferenceSampler, w.b, 0).rgb);

  float z = -(real.z * w.phase.x - imag.z * w.phase.y);
  float2 xy = real.xy * w.phase.y + imag.xy * w.phase.x;

  return significantWaveHeight * float3(xy, z);
}

float3 waveNormal(float2 world)
{
  WaveParameters w = calculateWaveParameters(world);

  float4 tmp_a = decodeTangents(Tangents.Sample(TangentsSampler, w.b));
  float4 tmp_b = decodeTangents(Tangents.Sample(TangentsSampler, 16.0 * w.b));

  float4 tmp = significantWaveHeight * (tmp_a + 0.5 * tmp_b);

  float2 rxy = (tmp.xz * w.phase.x - tmp.yw * w.phase.y); // Not completely correct, but seems to increase sharpness a bit
  float2 xy = tmp.xz * w.phase.y + tmp.yw * w.phase.x;

  float3 du = float3(1.0 + rxy.x, 0.0, xy.x);
  float3 dv = float3(0.0, 1.0 + rxy.y, xy.y);
  float3 m = cross(du, dv);

  return normalize(float3(waveDirection.x * m.x + waveDirection.y * m.y,
                          -waveDirection.y * m.x + waveDirection.x * m.y,
                          m.z));
}
