#include "AdaptivePlanarGrid.es20.glsl"

#ifdef COGS_BASIC_OCEAN_QUUX
#include "BasicOceanEffectQuux.es20.glsl"
#else
#include "BasicOceanEffect.es20.glsl"
#endif

float longitude(vec2 d)
{
  vec2 planeDir = normalize(-d);

  float camRelAzimuth = (0.5 / 3.14159) * acos(min(1.0, dot(planeDir, camPlaneDir())));

  if (planeDir.y * camPlaneDir().x - planeDir.x * camPlaneDir().y > 0.0) {
    camRelAzimuth = -camRelAzimuth;
  }

  return camRelAzimuth + camAzimuth();
}

float fresnel(vec3 view, vec3 normal)
{
  float refractionIndexRatio = 1.33;
  float R0 = pow(1.0 - refractionIndexRatio, 2.0) / pow(1.0 + refractionIndexRatio, 2.0);
  float cosAngle = 1.0 - clamp(dot(view, normal), 0.0, 1.0);

  float result = cosAngle * cosAngle;
  result = result * result;
  result = result * cosAngle;
  result = clamp((result * 1.0 - clamp(R0, 0.0, 1.0)) + R0, 0.0, 1.0);

  return result;
}

float fresnel2(vec3 view, vec3 normal)
{
  float eta = 0.66;
  float FresnelPower = 2.0;
  float F = ((1.0 - eta) * (1.0 - eta)) / ((1.0 + eta) * (1.0 + eta));

  float ratio = F + (1.0 - F) * (pow((1.0 - dot(view, normal)), FresnelPower));

  return ratio;
}

SurfaceOut surfaceFunction(VertexIn vertex)
{
  SurfaceOut Out = createSurfaceOut();

#ifdef COGS_BASIC_OCEAN_QUUX
  float roughness = 0.0;                //< How rough the sea should appear to be, value between 0.0 and 1.0.
  float foam = 0.0;
  vec3 n = waveNormal(foam, vertex.WorldPosition.xy, vertex.wavePosition.z, roughness);
#else
  float foam = 0.0;
  vec3 n = waveNormal(vertex.wavePosition.xy);
#endif
  n = n * vec3(2, 2, 1);

  vec3 v = normalize(inverseViewMatrix[3].xyz - vertex.WorldPosition.xyz);
  Out.Normal = normalize(n);

#if 0
  if (vertex.facing == false) {
    vec3 r = refract(v, n, 1.33);

    if (r.z < 0.02) {
      // Refraction into air
      return Out;
    }
  }
#endif

  float factor = -1.0;

//#ifdef COGS_SHADER_GL
  factor = 1.0;
//#endif

  vec4 reflectedColor = vec4(0, 0, 0, 1);
#if defined(COGS_BASIC_OCEAN_REFLECTION_PLANAR)
  vec2 screenPosition = vec2(0.5, factor * 0.5) * (vertex.screenPosition.xy / vertex.screenPosition.w) + 0.5;
  reflectedColor = reflectionBrightness() * texture2D(PlanarReflection, screenPosition + (n.xy * 0.05)) * vec4(0.7, 0.7, 0.7, 1.0);
  reflectedColor.rgb *= reflectionBrightness();
#elif defined(COGS_BASIC_OCEAN_REFLECTION_ENV_SKYBOX)
  vec3 refl = reflect(-v, n);
  reflectedColor = textureCubeLodEXT(environmentSky, refl.xzy, 0.0);
  reflectedColor.rgb *= reflectionBrightness() * environmentBrightness();
#elif defined(COGS_BASIC_OCEAN_REFLECTION_ENV_RADIANCE)
  vec3 refl = reflect(-v, n);
  reflectedColor = textureCubeLodEXT(environmentRadiance, refl.xzy, 0.0);
  reflectedColor.rgb *= reflectionBrightness() * environmentBrightness();
#endif

  bool submerged = false; // FIXME: Currently not exposed in ES2.
  if (!submerged) {
    float fresnelTerm = fresnel(v, n);
    Out.Color = vec4(waterColor().rgb * (1.0 - fresnelTerm), waterColor().a);
    Out.Emissive = (reflectedColor * fresnelTerm).rgb;
    Out.Specular = vec3(0.5, 0.5, 0.5);
    Out.SpecularPower = 20.0;
  }
  else {
    vec3 v_w = normalize(vertex.WorldPosition.xyz - inverseViewMatrix[3].xyz);

    float fresnelTerm = fresnel(v, -n);
    vec3 r = refract(v, n, 1.1);
    //Out.Color =  vec4(0, 0.01, 0.02, 1);
    float w = 0.0;
    if (r.z < 0.0) {
      w = abs(r.z);
    }
    Out.Color.rgb = w * reflectedColor.rgb;
    Out.Color.a = 1.0;
  }
  //Out.Color.rgb = (vec3)vertex.wavePosition.z;

  return Out;
}
