
#if defined(COGS_BASIC_TERRAIN_ELEVATION_IN_RED)
float elevation(float2 texpos)
{
  return Elevation.SampleLevel(ElevationSampler, texpos, 0.0).r;
}
#elif defined(COGS_BASIC_TERRAIN_ELEVATION_IN_ALPHA)
float elevation(float2 texpos)
{
  return Elevation.SampleLevel(ElevationSampler, texpos, 0.0).a;
}
#elif defined(COGS_BASIC_TERRAIN_ELEVATION_IN_RED_GREEN_BLUE)
float elevation(float2 texpos)
{
  float3 value = Elevation.SampleLevel(ElevationSampler, texpos, 0.0).rgb;
  return 256.0 * 256.0 * 256.0 * value.r + 256.0 * 256.0 * value.g + 256.0 * value.b;
}
#endif

float3 vibranceFilter(float3 color)
{
  if (vibrance == 1.0) return color;

  float ma = max(color.r, max(color.g, color.b));
  float mi = min(color.r, min(color.g, color.b));
  float saturation = ma - mi;
  float scale = pow(abs(saturation), abs(vibrance)) / max(saturation, 0.01);

  float3 white = (float3)(1.0 / sqrt(3.0));
  float3 grey = dot(color, white) * white;

  return grey + scale * (color - grey);
}

SurfaceOut surfaceFunction(VertexIn vertex)
{
  SurfaceOut Out = (SurfaceOut)0;

#ifdef COGS_BASIC_TERRAIN_HAS_NODATA
  if (vertex.texCoord.z == noDataValue) discard;
#endif

#if defined(COGS_BASIC_TERRAIN_SOLID)
  float4 c = diffuseColor;
#elif defined(COGS_BASIC_TERRAIN_TEXTURE_RGB)
  float4 c = float4(Color.Sample(ColorSampler, vertex.texCoord.xy).rgb, 1);
  c.rgb = diffuseColor.rgb * vibranceFilter(c.rgb);
#elif defined(COGS_BASIC_TERRAIN_TEXTURE_RGBA)
  float4 c = Color.Sample(ColorSampler, vertex.texCoord.xy);
  c.rgb = diffuseColor.rgb * vibranceFilter(c.rgb);
#endif

#ifdef COGS_BASIC_TERRAIN_ELEVATION_TO_ALPHA
  {
    float e = elevation(vertex.texCoord.xy);
    float3 ctr = elevationToAlphaTransform;
    c.a = pow(saturate(ctr.y * (e - ctr.x)), ctr.z);
  }
#endif

  float dh_du = elevation(vertex.texCoord.xy + float2(elevationTransform.z, 0))
              - elevation(vertex.texCoord.xy - float2(elevationTransform.z, 0));
  float dh_dv = elevation(vertex.texCoord.xy + float2(0, elevationTransform.w))
              - elevation(vertex.texCoord.xy - float2(0, elevationTransform.w));

  Out.Normal = normalize(cross(float3(normalTransform.x, 0, dh_du),
                               float3(0, normalTransform.y, dh_dv)));
  Out.Color = float4(c.rgb, c.a);
  Out.Specular = float3(0.0, 0.0, 0.0);
  Out.SpecularPower = 0.f;

  return Out;
}
