#include "Default/DefaultCommon.es20.glsl"

SurfaceOut surfaceFunction(in VertexIn In)
{
  SurfaceOut Out = createSurfaceOut();

  UpdateMaterialProperties(In, Out);

#ifdef COGS_DEFAULT_MATERIAL_TEXTURED
  vec4 textureColor = texture2D(diffuseMap, In.TexCoord * diffuseMapScale());

  Out.Color *= textureColor;
#endif

#ifdef DEFAULT_MATERIAL_INSTANCED_COLORS
  Out.Color *= In.InstanceColor;
#endif

#ifdef DEFAULT_MATERIAL_ENABLE_LIGHTING

#if defined(DEFAULT_MATERIAL_OBJECT_SPACE_NORMAL_MAP)
  Out.Normal = normalize(texture2D(normalMap, In.TexCoord * normalMapScale()).rgb);
#elif defined(BASE_MATERIAL_VERTEX_NORMAL0)
  vec3 normal = normalize(In.Normal);
#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  vec3 normalColor = 2.0 * texture2D(normalMap, In.TexCoord * normalMapScale()).rgb - 1.0;

  vec3 tangent = normalize(In.Tangent);
  vec3 biTangent = cross(normal, tangent);

  vec3 bumpNormal = normalize(normalColor.z * normal + normalColor.x * tangent + normalColor.y * biTangent);
  normal = normalize(mix(normal, bumpNormal, normalMapFactor()));
#endif
  Out.Normal = In.Facing ? normal : -normal;
#endif

#endif

#ifdef DEFAULT_MATERIAL_SPECULAR_MAP
  vec3 specularColor = texture2D(specularMap, In.TexCoord * specularMapScale()).rgb;

  Out.SpecularPower = 20.0 + max(specularColor.r, max(specularColor.g, specularColor.b)) * 80.0;
  Out.Specular = specularColor.rgb;
#endif

  return Out;
}
