#include "Default/DefaultCommon.es30.glsl"

VertexOut vertexFunction(VertexIn In)
{
  VertexOut Out = createVertexOut();

#ifdef COGS_VARIANT_SKINNED_BASIC
  float w0 = In.BoneWeights.x;
  int i0 = In.BoneIndexes.x;

  float w1 = In.BoneWeights.y;
  int i1 = In.BoneIndexes.y;

  float w2 = In.BoneWeights.z;
  int i2 = In.BoneIndexes.z;

  float w3 = In.BoneWeights.w;
  int i3 = In.BoneIndexes.w;

  vec4 position = w0 * boneTransforms[i0] * In.Position;
  position     += w1 * boneTransforms[i1] * In.Position;
  position	   += w2 * boneTransforms[i2] * In.Position;
  position	   += w3 * boneTransforms[i3] * In.Position;

#if defined(DEFAULT_MATERIAL_ENABLE_LIGHTING) && defined(BASE_MATERIAL_VERTEX_NORMAL0)
  vec3 normal = w0 * mat3(boneTransforms[i0]) * In.Normal;
  normal     += w1 * mat3(boneTransforms[i1]) * In.Normal;
  normal     += w2 * mat3(boneTransforms[i2]) * In.Normal;
  normal     += w3 * mat3(boneTransforms[i3]) * In.Normal;

#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  vec3 tangent = w0 * mat3(boneTransforms[i0]) * In.Tangent;
  tangent     += w1 * mat3(boneTransforms[i1]) * In.Tangent;
  tangent     += w2 * mat3(boneTransforms[i2]) * In.Tangent;
  tangent     += w3 * mat3(boneTransforms[i3]) * In.Tangent;
#endif
#endif

#else
  vec4 position = In.Position;

#ifdef DEFAULT_MATERIAL_INSTANCED_POSITIONS
  position.xyz += In.InstancePosition.xyz;
#endif

#if defined(DEFAULT_MATERIAL_ENABLE_LIGHTING) && defined(BASE_MATERIAL_VERTEX_NORMAL0)
  vec3 normal = In.Normal;
#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  vec3 tangent = In.Tangent;
#endif
#endif
#endif

  Out.WorldPosition = worldMatrix * position;

#ifdef DEFAULT_MATERIAL_INSTANCED_WORLD_POSITIONS
  Out.WorldPosition.xyz += In.InstanceWorldPosition.xyz;
#endif
  Out.Position = worldToClipMatrix * Out.WorldPosition;

#if defined(DEFAULT_MATERIAL_ENABLE_LIGHTING) && defined(BASE_MATERIAL_VERTEX_NORMAL0)
  Out.Normal = mat3(worldMatrix) * normal;
#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
 Out.Tangent = mat3(worldMatrix) * tangent;
#endif
#endif

#ifdef COGS_DEFAULT_MATERIAL_TEXTURED
  Out.TexCoord = In.TexCoord;
#endif

#ifdef DEFAULT_MATERIAL_INSTANCED_COLORS
  Out.InstanceColor = In.InstanceColor;
#endif

  return Out;
}
