#include "Default/DefaultCommon.hlsl"

VertexOut vertexFunction(VertexIn In)
{
  VertexOut Out = (VertexOut)0;

#ifdef COGS_VARIANT_SKINNED_BASIC
  const float w0 = In.BoneWeights.x;
  const int i0 = In.BoneIndexes.x;

  const float w1 = In.BoneWeights.y;
  const int i1 = In.BoneIndexes.y;

  const float w2 = In.BoneWeights.z;
  const int i2 = In.BoneIndexes.z;

  const float w3 = In.BoneWeights.w;
  const int i3 = In.BoneIndexes.w;

  float4 position = w0 * mul(In.Position, boneTransforms[i0]);
  position += w1 * mul(In.Position, boneTransforms[i1]);
  position += w2 * mul(In.Position, boneTransforms[i2]);
  position += w3 * mul(In.Position, boneTransforms[i3]);

#if defined(DEFAULT_MATERIAL_ENABLE_LIGHTING) && defined(BASE_MATERIAL_VERTEX_NORMAL0)
  float3 normal = w0 * mul(In.Normal, (float3x3)boneTransforms[i0]);
  normal += w1 * mul(In.Normal, (float3x3)boneTransforms[i1]);
  normal += w2 * mul(In.Normal, (float3x3)boneTransforms[i2]);
  normal += w3 * mul(In.Normal, (float3x3)boneTransforms[i3]);

#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  float3 tangent = w0 * mul(In.Tangent, (float3x3)boneTransforms[i0]);
  tangent += w1 * mul(In.Tangent, (float3x3)boneTransforms[i1]);
  tangent += w2 * mul(In.Tangent, (float3x3)boneTransforms[i2]);
  tangent += w3 * mul(In.Tangent, (float3x3)boneTransforms[i3]);
#endif
#endif

#else
  float4 position = In.Position;

#ifdef DEFAULT_MATERIAL_INSTANCED_POSITIONS
  position.xyz += In.InstancePosition.xyz;
#endif

#if defined(DEFAULT_MATERIAL_ENABLE_LIGHTING) && defined(BASE_MATERIAL_VERTEX_NORMAL0)
  float3 normal = In.Normal;
#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  float3 tangent = In.Tangent;
#endif
#endif
#endif

  Out.WorldPosition = mul(position, worldMatrix);

#ifdef DEFAULT_MATERIAL_INSTANCED_WORLD_POSITIONS
  Out.WorldPosition.xyz += In.InstanceWorldPosition.xyz;
#endif
  Out.Position = mul(Out.WorldPosition, worldToClipMatrix);

#if defined(DEFAULT_MATERIAL_ENABLE_LIGHTING) && defined(BASE_MATERIAL_VERTEX_NORMAL0)
  Out.Normal = mul(normal, (float3x3)worldMatrix);
#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  Out.Tangent = mul(tangent, (float3x3)worldMatrix);
#endif
#endif

#ifdef COGS_DEFAULT_MATERIAL_TEXTURED
  Out.TexCoord = In.TexCoord;
#endif

#ifdef DEFAULT_MATERIAL_INSTANCED_COLORS
  Out.InstanceColor = In.InstanceColor;
#endif

  return Out;
}
