#version 300 es
precision highp float;
precision highp int;

#include "Engine/Common.es30.glsl"
#include "Engine/PhongLighting.es30.glsl"

sampler2D colorTexture;
sampler2D emissiveTexture;
sampler2D specularTexture;
sampler2D normalTexture;
sampler2D depthTexture;

in vec2 NormalizedCoords;

out vec4 Out_Color;

void main()
{
  vec4 color = texelFetch(colorTexture, ivec2(gl_FragCoord.xy), 0);
  vec4 emissive = texelFetch(emissiveTexture, ivec2(gl_FragCoord.xy), 0);
  vec4 specular = texelFetch(specularTexture,ivec2(gl_FragCoord.xy), 0);
  vec4 normal = texelFetch(normalTexture, ivec2(gl_FragCoord.xy), 0);
  float depth = texelFetch(depthTexture, ivec2(gl_FragCoord.xy), 0);

  //if (color.a == 0.0) discard;

  vec4 viewSpacePosition = inverseProjectionMatrix * vec4(NormalizedCoords, depth, 1);
  viewSpacePosition /= viewSpacePosition.w;

  vec4 worldPosition = inverseViewMatrix * viewSpacePosition;
  worldPosition.xyz *= (1.0 / worldPosition.w);

  SurfaceLighting surface;
  surface.Color = color;
  surface.Normal = normal.xyz;
  surface.Specular = specular.xyz;
  surface.ViewSpacePosition = viewSpacePosition.xyz;
  surface.SpecularPower = normal.w;
  surface.Emissive = emissive.rgb;
  surface.LightModel = int(255.0 * specular.w);
  surface.WorldPosition = worldPosition;

  Out_Color = vec4(DoLighting(surface, gl_FragCoord).rgb, 1.0);
  gl_FragDepth = depth;
}
