vec4 ShowNormals(SurfaceOut surface)
{
  return vec4(vec3(0.5, 0.5, 0.5) + surface.Normal * 0.5, 1.0);
}

vec4 ShowColor(SurfaceOut surface)
{
  return surface.Color;
}

#include "Engine/PhongLighting.es20.glsl"

#ifdef COGS_DITHER_FORWARD
//#include "BlueNoise.hlsl"
#endif

#if defined(COGS_TONEMAP_FILMIC)
// Filmic tonemapper (aka Uncharted2)
//
// http://filmicworlds.com/blog/filmic-tonemapping-operators/

vec3 filmicToneMapOperator(vec3 c)
{
    return ((c*(0.15*c+0.10*0.50)+0.20*0.02)/(c*(0.15*c+0.50)+0.20*0.30))-0.02/0.30;
}

vec3 tonemap(vec3 c)
{
    // Exposure should be divided by 3 to match other curves
    return (1.0/filmicToneMapOperator(vec3(11.2)))*filmicToneMapOperator((1.0/3.0) * 2.0 * exposure() * c.rgb);
}

#elif defined(COGS_TONEMAP_ACES_TRIBAND)
// ACES tri-band tonemapper
//
// https://github.com/TheRealMJP/BakingLab/blob/master/BakingLab/ACES.hlsl

vec3 tonemap(vec3 x)
{
    const mat3 ACESInputMat = mat3(0.59719, 0.07600, 0.02840,   // col 0
                                   0.35458, 0.90834, 0.13383,   // col 1 ..
                                   0.04823, 0.01566, 0.83777);

    const mat3 ACESOutputMat = mat3( 1.60475, -0.10208, -0.00327, 
                                    -0.53108,  1.10813, -0.07276,  
                                    -0.07367, -0.00605, 1.07602);
    vec3 t0 = ACESInputMat * 2.0 * exposure() * x;
    vec3 t1 = t0 * (           t0 + 0.0245786) - 0.000090537;
    vec3 t2 = t0 * (0.983729 * t0 + 0.4329510) + 0.238081;
    vec3 t3 = t1/t2;
    vec3 t4 = ACESOutputMat * t3;
    return t4;
}

#elif defined(COGS_TONEMAP_ACES_LUMINANCE)
// ACES luminance-only
//
//https://knarkowicz.wordpress.com/2016/01/06/aces-filmic-tone-mapping-curve/

vec3 tonemap(vec3 x)
{
    x = exposure() * x;
    const float a = 2.51;
    const float b =0.03;
    const float c =2.43;
    const float d =0.59;
    const float e =0.14;
    return ((x*(a*x+b))/(x*(c*x+d)+e));
}
#endif


void main(/*VertexIn In, float4 screenPos : SV_Position*/)
{
  vec4 screenPos = gl_FragCoord;
  //PsOutput Out = (PsOutput)0;

  VertexIn In = importVaryings();
  SurfaceOut surface = SURFACE_FUNCTION(In);

#ifdef COGS_ALPHA_TEST_DISCARD
  if (surface.Color.a < alphaThreshold()) discard;
#endif

  SurfaceLighting surfaceLighting;
  surfaceLighting.Color = surface.Color;
  surfaceLighting.Emissive = surface.Emissive;
  surfaceLighting.Normal = surface.Normal;
  surfaceLighting.ViewSpacePosition = (1.0/In.ViewSpacePosition.w)*In.ViewSpacePosition.xyz;
  surfaceLighting.SpecularPower = surface.SpecularPower;
  surfaceLighting.Specular = surface.Specular;
  surfaceLighting.WorldPosition = In.WorldPosition;

  vec4 color;
#ifdef COGS_DEBUG_VIEW_NORMALS
  color = vec4(surface.Normal * 0.5 + 0.5, 1);
#elif defined(COGS_LIGHT_MODEL_NONE)
  color = vec4(1, 1, 1, 1);
#else
//  surfaceLighting.LightModel = GetMaterialId() | 32;
  vec3 light = DoLighting(surfaceLighting, screenPos).rgb;

#if defined(COGS_TONEMAP_FILMIC) || defined(COGS_TONEMAP_ACES_LUMINANCE) || defined(COGS_TONEMAP_ACES_TRIBAND)
  light.rgb = tonemap(light.rgb);
#endif

  color = vec4(light, surfaceLighting.Color.a);
#endif



//#ifdef COGS_DITHER_FORWARD
//  float3 dither = (blueNoise3(int2(screenPos.xy))*2.0-1.0)*(1.0/255);
//  color.rgb = saturate(color.rgb + dither);
//#endif
//

  if(outputSRGB()) {
    color.rgb = clamp(color.rgb, vec3(0.0), vec3(1.0));
    color.rgb = mix(pow(color.rgb, vec3(1.0/2.4)),
                    12.92 * color.rgb,
                    vec3(lessThan(color.rgb, vec3(0.0031308))));
  }
  gl_FragColor = color;

//#ifdef COGS_OUTPUT_NORMALS
//  Out.Normal = surface.Normal;
//#endif
}
