TextureCubeArray cubeShadowMap;

const static float3 samplingDisk[20] = {
  { 1, 1, 1 }, {  1, -1, 1 }, { -1, -1,  1 }, { -1, 1,  1 },
  { 1, 1,-1 }, {  1, -1,-1 }, { -1, -1, -1 }, { -1, 1, -1 },
  { 1, 1, 0 }, {  1, -1, 0 }, { -1, -1,  0 }, { -1, 1,  0 },
  { 1, 0, 1 }, { -1,  0, 1 }, {  1,  0, -1 }, { -1, 0, -1 },
  { 0, 1, 1 }, {  0, -1, 1 }, {  0, -1, -1 }, {  0, 1, -1 },
};

float vectorToDepth(in float3 v, in float n, in float f)
{
  float3 absVector = abs(v);
  float localViewZ = max(absVector.x, max(absVector.y, absVector.z));

  float normalizedZ = (f + n) / (f - n) + ((-2 * f * n) / (f - n)) * (1.0 / localViewZ);
#ifdef COGS_REVERSE_DEPTH
  normalizedZ = -normalizedZ;
#endif

  return normalizedZ*0.5 + 0.5;
}

float calculatePointLightShadow(in float4 worldLightPos, in float3 worldPosition, in float3 normal, in int index)
{
  ShadowData shadow = shadows[index];

  if (!shadow.enabled) return 1;

  float3 toLight = worldLightPos.xyz - worldPosition;
  float3 lightDir = normalize(toLight);

  //float3 worldSpaceNormal = normal;
  //float normalSlopeScale = saturate(1 - dot(worldSpaceNormal, lightDir));

  //worldPosition.xyz += normalSlopeScale * worldSpaceNormal * shadow.normalShadowBias;

  toLight = worldLightPos.xyz - worldPosition;
  lightDir = normalize(toLight);

  float depth = vectorToDepth(toLight, shadow.nearSplits[0], shadow.farSplits[0]);
  depth = saturate(depth-shadow.shadowSampleBias);

  float3 dimensions;
  cubeShadowMap.GetDimensions(dimensions.x, dimensions.y, dimensions.z);
  float texelSize = 1.0 / dimensions.x;

  float shadowFactor = 0;

#if defined(COGS_SOFT_SHADOWS_HIGH)
  #define DISKSAMPLES 20
#elif defined(COGS_SOFT_SHADOWS_LOW)
  #define DISKSAMPLES 8
#else
  #define DISKSAMPLES 1
#endif

#if DISKSAMPLES > 1
  for (int j = 0; j < DISKSAMPLES; j++) {
    float3 shadowCoords = -lightDir + samplingDisk[j] * texelSize;

#ifdef GLSLANG
    shadowFactor += cubeShadowMap.SampleCmp(cascadedShadowSampler, float4(shadowCoords.xzy, cascadeOffsets[index]), depth);
#else
    shadowFactor += cubeShadowMap.SampleCmpLevelZero(cascadedShadowSampler, float4(shadowCoords.xzy, cascadeOffsets[index]), depth);
#endif
  }

  shadowFactor *= (1.0 / DISKSAMPLES);
#else
#ifdef GLSLANG
  shadowFactor = cubeShadowMap.SampleCmp(cascadedShadowSampler, float4(-lightDir.xzy, cascadeOffsets[index]), depth);
#else
  shadowFactor = cubeShadowMap.SampleCmpLevelZero(cascadedShadowSampler, float4(-lightDir.xzy, cascadeOffsets[index]), depth);
#endif

#endif

  return shadowFactor;
}
