#include "BasicOceanEffect.es20.glsl"
#include "AdaptivePlanarGrid.es20.glsl"

attribute vec3 a_POSITION0;

void main()
{
  // Snap edge vertices to neighbouring grids
  vec2 pos = snapAdaptivePlanarGridEdges(a_POSITION0.xyz).xy;

  // Move and scale patch to its position in world space
  vec3 world = (worldMatrix * vec4(pos, 0, 1)).xyz;

  // Calcualte coordinates in wave space.
  vec2 waveCoordinates = getTexCoordWorld(pos.xy);
  
  // Determine pixel size of significant waveheight, which we use to control LoD
  vec4 a0 = (projectionMatrix * (viewMatrix * vec4(world, 1.0)));
  vec4 a1 = (projectionMatrix * (viewMatrix * vec4(world + cameraYAxis.xyz, 1.0)));
  float lod = significantWaveHeight*cameraYAxis.w*((a1.y / a1.w) - (a0.y / a0.w));

  // And we only displace if we are relatively close to the camera
  vec3 surfacePositionWorld = world;
  if (lod > 0.0) {
    float blend = clamp(lod - 2.0, 0.0, 1.0);
    surfacePositionWorld += 1.0 * waveDisplace(waveCoordinates);
  }
  
  // Output results

  vertex_displacedPositionWorld = surfacePositionWorld;
  vertex_undisplacedPositionWorld = world;
  vertex_wavePosition = waveCoordinates;
  vertex_screenPosition = worldToClipMatrix * vec4(surfacePositionWorld, 1.0);
  
  gl_Position = vertex_screenPosition;
}
