#include "Engine/Common.hlsl"

Texture2D depth4xTexture;
Texture2D occlusion4xTexture;

SamplerState linearSampler;

struct VertexIn
{
  float4 Position : SV_Position;
  float4 ScreenPosition : TEXCOORD0;
  float2 TexCoords : TEXCOORD1;
};

static float gaussWeightsSigma1[7] =
{
  0.00598f,
  0.060626f,
  0.241843f,
  0.383103f,
  0.241843f,
  0.060626f,
  0.00598f
};

static float gaussWeightsSigma3[7] =
{
  0.106595f,
  0.140367f,
  0.165569f,
  0.174938f,
  0.165569f,
  0.140367f,
  0.106595f
};

float4 main(VertexIn In) : SV_Target
{
  float2 pixelSize;
  depth4xTexture.GetDimensions(pixelSize.x, pixelSize.y);

#ifdef AO_BLUR_Y
  float2 pixelOffset = float2(0, 1.0 / pixelSize.y);
#else
  float2 pixelOffset = float2(1.0 / pixelSize.x, 0);
#endif

  float sum = 0.0f;
  float weightsSum = 0.0f;

  float depth = depth4xTexture.Sample(linearSampler, In.TexCoords).x;

  for (int i = -3; i <= 3; i++) {
    float2 sampleTexCoord = In.TexCoords + i * pixelOffset;
    float sampleDepth = depth4xTexture.Sample(linearSampler, sampleTexCoord).x;

    float depthsDiff = 0.1f * abs(depth - sampleDepth);
    depthsDiff *= depthsDiff;

    float weight = 1.0f / (depthsDiff + 0.001f);
    weight *= gaussWeightsSigma3[3 + i];

    sum += weight * occlusion4xTexture.Sample(linearSampler, sampleTexCoord).x;
    weightsSum += weight;
  }

  return sum / weightsSum;
}
