VertexOut vertexFunction(VertexIn In)
{
  VertexOut Out = createVertexOut();

#ifdef COGS_VARIANT_SKINNED_BASIC
  float w0 = In.BoneWeights.x;
  int i0 = In.BoneIndexes.x;

  float w1 = In.BoneWeights.y;
  int i1 = In.BoneIndexes.y;

  float w2 = In.BoneWeights.z;
  int i2 = In.BoneIndexes.z;

  float w3 = In.BoneWeights.w;
  int i3 = In.BoneIndexes.w;

  vec4 position = w0 * boneTransforms[i0] * In.Position;
  position     += w1 * boneTransforms[i1] * In.Position;
  position     += w2 * boneTransforms[i2] * In.Position;
  position     += w3 * boneTransforms[i3] * In.Position;

  vec3 normal = w0 * mat3(boneTransforms[i0]) * In.Normal;
  normal     += w1 * mat3(boneTransforms[i1]) * In.Normal;
  normal     += w2 * mat3(boneTransforms[i2]) * In.Normal;
  normal     += w3 * mat3(boneTransforms[i3]) * In.Normal;

#ifdef COGS_STANDARD_MATERIAL_NORMAL_MAP

#if BASE_MATERIAL_VERTEX_TANGENT0 == COGS_FLOAT4
  float tangentW = In.Tangent.w;
#else
  float tangentW = 1.0;
#endif

  vec3 tangent = w0 * mat3(boneTransforms[i0]) * In.Tangent.xyz;
  tangent     += w1 * mat3(boneTransforms[i1]) * In.Tangent.xyz;
  tangent     += w2 * mat3(boneTransforms[i2]) * In.Tangent.xyz;
  tangent     += w3 * mat3(boneTransforms[i3]) * In.Tangent.xyz;

#ifdef COGS_STANDARD_MATERIAL_HAS_BITANGENT
  vec3 bitangent = w0 * mat3(boneTransforms[i0]) * In.Bitangent;
  bitangent     += w1 * mat3(boneTransforms[i1]) * In.Bitangent;
  bitangent     += w2 * mat3(boneTransforms[i2]) * In.Bitangent;
  bitangent     += w3 * mat3(boneTransforms[i3]) * In.Bitangent;
#endif
#endif
#else
  vec4 position = In.Position;
  vec3 normal = In.Normal;
#ifdef COGS_STANDARD_MATERIAL_NORMAL_MAP
  vec3 tangent = In.Tangent.xyz;

#ifdef COGS_STANDARD_MATERIAL_HAS_BITANGENT
  vec3 bitangent = In.Bitangent;
#endif
#endif
#endif

  Out.WorldPosition = worldMatrix * position;
  Out.Position = worldToClipMatrix * Out.WorldPosition;

  Out.Normal = mat3(worldMatrix) * normal;
#ifdef COGS_STANDARD_MATERIAL_NORMAL_MAP
  Out.Tangent = mat3(worldMatrix) * tangent;
#ifdef COGS_STANDARD_MATERIAL_HAS_BITANGENT
  Out.Bitangent = mat3(worldMatrix) * bitangent;
#endif
#endif

#ifdef COGS_STANDARD_MATERIAL_TEXTURED
  Out.TexCoord0 = In.TexCoord0;
#endif

#ifdef COGS_STANDARD_MATERIAL_SECONDARY_UV
  Out.TexCoord1 = In.TexCoord1;
#endif

#ifdef COGS_STANDARD_MATERIAL_ALBEDO_PER_VERTEX
  Out.Albedo = In.Albedo;
#endif

  return Out;
}
